// interface part

#ifdef __cplusplus
extern "C" {
#endif

void init(int *n, int *s, int *f);
void getXYM(int r, int *x, int *y, int *m);
void getQ(int r, int *q);
void put(int r);
void done();

#ifdef __cplusplus
}
#endif

// implementation part
// should be in a library, but this is not supported by the grading server

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define __TEELIB_MAXN 7000
#define __TEELIB_MAXM 5
#define __TEELIB_MAXT 20000

struct __TEELIB_STATE {
	int ic; // init counter
	int gc; // get counter
	int pc; // put counter
	int dc; // done counter
	FILE *fi; // input file
	FILE *fo; // output file
	int n, s, f; // n, s, f from input
	int x[__TEELIB_MAXN]; // x[r] from input
	int y[__TEELIB_MAXN]; // y[r] from input
	int m[__TEELIB_MAXN]; // m[r] from input
	int q[__TEELIB_MAXN][__TEELIB_MAXM]; // q[r][i] from input
	int qc[__TEELIB_MAXN]; // q[r] read counter
	int p[__TEELIB_MAXN]; // path
	int t; // total streets
} __teelib_state = {0, 0, 0, 0};

void __teelib_error(const char *msg) {
	fprintf(stderr, "%s\n", msg);
	exit(EXIT_FAILURE);
}

void init(int *n, int *s, int *f) {
	int i, j;

	if (__teelib_state.ic != 0)
		__teelib_error("Second call to init()");
	++__teelib_state.ic;

	__teelib_state.fi = fopen("tee.sis", "rt");
	if (__teelib_state.fi == NULL)
		__teelib_error("Could not open input file");

	__teelib_state.fo = fopen("tee.val", "wt");
	if (__teelib_state.fo == NULL)
		__teelib_error("Could not open output file");

	if (fscanf(__teelib_state.fi, "%d %d %d", &__teelib_state.n, &__teelib_state.s, &__teelib_state.f) != 3)
		__teelib_error("Error reading N, S, F from input file");
	if (__teelib_state.n < 1 || __teelib_state.n > __TEELIB_MAXN)
		__teelib_error("Invalid N in input file");
	if (__teelib_state.s < 1 || __teelib_state.s > __teelib_state.n)
		__teelib_error("Invalid S in input file");
	if (__teelib_state.f < 1 || __teelib_state.f > __teelib_state.n)
		__teelib_error("Invalid F in input file");

	for (i = 0; i < __teelib_state.n; ++i) {
		if (fscanf(__teelib_state.fi, "%d %d %d", &__teelib_state.x[i], &__teelib_state.y[i], &__teelib_state.m[i]) != 3)
			__teelib_error("Error reading X[r], Y[r], M[r] from input file");
		if (__teelib_state.m[i] < 0 || __teelib_state.m[i] > __TEELIB_MAXM)
			__teelib_error("Invalid M[r] in input file");
		__teelib_state.t += __teelib_state.m[i];
		for (j = 0; j < __teelib_state.m[i]; ++j) {
			if (fscanf(__teelib_state.fi, "%d", &__teelib_state.q[i][j]) != 1)
				__teelib_error("Error reading Q[r][i] from input file");
			if (__teelib_state.q[i][j] < 1 || __teelib_state.q[i][j] > __teelib_state.n)
				__teelib_error("Invalid Q[r][i] in input file");
		}
	}
	if (__teelib_state.t > __TEELIB_MAXT)
		__teelib_error("Too many streets in input file");

	*n = __teelib_state.n;
	*s = __teelib_state.s;
	*f = __teelib_state.f;

	fprintf(__teelib_state.fo, "init() -> %d %d %d\n", __teelib_state.n, __teelib_state.s, __teelib_state.f);
}

void getXYM(int r, int *x, int *y, int *m) {
	if (__teelib_state.ic != 1)
		__teelib_error("Call to getXYM() before init()");
	if (__teelib_state.dc != 0)
		__teelib_error("Call to getXYM() after done()");
	++__teelib_state.gc;

	if (r < 1 || r > __teelib_state.n)
		__teelib_error("Invalid call to getXYM()");

	*x = __teelib_state.x[r - 1];
	*y = __teelib_state.y[r - 1];
	*m = __teelib_state.m[r - 1];

	fprintf(__teelib_state.fo, "getXYM(%d) -> %d %d %d\n", r, *x, *y, *m);
}

void getQ(int r, int *q) {
	if (__teelib_state.ic != 1)
		__teelib_error("Call to getQ() before init()");
	if (__teelib_state.dc != 0)
		__teelib_error("Call to getQ() after done()");
	++__teelib_state.gc;

	if (r < 1 || r > __teelib_state.n)
		__teelib_error("Invalid call to getQ()");
	if (__teelib_state.qc[r - 1] >= __teelib_state.m[r - 1])
		__teelib_error("Too many calls to getQ()");

	*q = __teelib_state.q[r - 1][__teelib_state.qc[r - 1]];
	++__teelib_state.qc[r - 1];

	fprintf(__teelib_state.fo, "getQ(%d) -> %d\n", r, *q);
}

void put(int r) {
	if (__teelib_state.ic != 1)
		__teelib_error("Call to put() before init()");
	if (__teelib_state.dc != 0)
		__teelib_error("Call to put() after done()");
	if (__teelib_state.pc >= __teelib_state.n)
		__teelib_error("Too many calls to put()");

	if (r < 1 || r > __teelib_state.n)
		__teelib_error("Invalid call to put()");
	__teelib_state.p[__teelib_state.pc] = r;
	++__teelib_state.pc;

	fprintf(__teelib_state.fo, "put(%d)\n", r);
}

void done() {
	int i, j, r, q;
	double dx, dy, d;

	if (__teelib_state.ic != 1)
		__teelib_error("Call to done() before init()");
	if (__teelib_state.dc != 0)
		__teelib_error("Second call to done()");
	++__teelib_state.dc;

	r = -1;
	d = 0.0;
	for (i = 0; i < __teelib_state.pc; ++i) {
		if (i == 0 && __teelib_state.p[i] != __teelib_state.s)
			__teelib_error("Path does not start from S");
		if (i > 0) {
			for (j = 0; j < __teelib_state.m[r - 1]; ++j)
				if (__teelib_state.q[r - 1][j] == __teelib_state.p[i])
					break;
			if (j >= __teelib_state.m[r - 1])
				__teelib_error("Path uses non-existent street");
			q = __teelib_state.p[i];
			dx = __teelib_state.x[r - 1] - __teelib_state.x[q - 1];
			dy = __teelib_state.y[r - 1] - __teelib_state.y[q - 1];
			d += sqrt(dx * dx + dy * dy);
		}
		r = __teelib_state.p[i];
	}
	if (r != __teelib_state.f)
		__teelib_error("Path does not finish at F");

	fprintf(__teelib_state.fo, "done() -> distance %0.4lf, %d queries\n", d, __teelib_state.gc);

	fclose(__teelib_state.fi);
	fclose(__teelib_state.fo);
}
