// interface part

procedure init(var n, s, f : longint); forward;
procedure getXYM(r : longint; var x, y, m : longint); forward;
procedure getQ(r : longint; var q : longint); forward;
procedure put(r : longint); forward;
procedure done(); forward;

// implementation part
// should be in a unit, but this is not supported by the grading server

const __teelib_maxn = 7000;
const __teelib_maxm = 5;
const __teelib_maxt = 20000;

var __teelib_state : record
	ic : longint; // init counter
	gc : longint; // get counter
	pc : longint; // put counter
	dc : longint; // done counter
	fi : text; // input file
	fo : text; // output file
	n, s, f : longint; // n, s, f from input
	x, y, m : array [1..__teelib_maxn] of longint; // x[r], y[r], m[r] from input
	q : array [1..__teelib_maxn, 1..__teelib_maxm] of longint; // q[r][i] from input
	qc : array [1..__teelib_maxn] of longint; // q[r] read counter
	p : array [1..__teelib_maxn] of longint; // path
	t : longint; // total streets
end;

procedure __teelib_error(msg : string);
begin
	writeln(stderr, msg);
	halt(1);
end;

procedure init(var n, s, f : longint);
var i, j : longint;
begin
	if __teelib_state.ic <> 0 then
		__teelib_error('Second call to init()');
	inc(__teelib_state.ic);

	assign(__teelib_state.fi, 'tee.sis'); reset(__teelib_state.fi);
	if ioresult <> 0 then
		__teelib_error('Could not open input file');

	assign(__teelib_state.fo, 'tee.val'); rewrite(__teelib_state.fo);
	if ioresult <> 0 then
		__teelib_error('Could not open output file');

	read(__teelib_state.fi, __teelib_state.n, __teelib_state.s, __teelib_state.f);
	if ioresult <> 0 then
		__teelib_error('Error reading N, S, F from input file');
	if (__teelib_state.n < 1) or (__teelib_state.n > __teelib_maxn) then
		__teelib_error('Invalid N in input file');
	if (__teelib_state.s < 1) or (__teelib_state.s > __teelib_state.n) then
		__teelib_error('Invalid S in input file');
	if (__teelib_state.f < 1) or (__teelib_state.f > __teelib_state.n) then
		__teelib_error('Invalid F in input file');

	for i := 1 to __teelib_state.n do begin
		read(__teelib_state.fi, __teelib_state.x[i], __teelib_state.y[i], __teelib_state.m[i]);
		if ioresult <> 0 then
			__teelib_error('Error reading X[r], Y[r], M[r] from input file');
		if (__teelib_state.m[i] < 0) or (__teelib_state.m[i] > __teelib_maxm) then
			__teelib_error('Invalid M[r] in input file');
		inc(__teelib_state.t, __teelib_state.m[i]);
		for j := 1 to __teelib_state.m[i] do begin
			read(__teelib_state.fi, __teelib_state.q[i][j]);
			if ioresult <> 0 then
				__teelib_error('Error reading Q[r][i] from input file');
			if (__teelib_state.q[i][j] < 1) or (__teelib_state.q[i][j] > __teelib_state.n) then
				__teelib_error('Invalid Q[r][i] in input file');
		end;
	end;
	if __teelib_state.t > __teelib_maxt then
		__teelib_error('Too many streets in input file');

	n := __teelib_state.n;
	s := __teelib_state.s;
	f := __teelib_state.f;

	writeln(__teelib_state.fo, 'init() -> ', __teelib_state.n, ' ', __teelib_state.s, ' ', __teelib_state.f);
end;

procedure getXYM(r : longint; var x, y, m : longint);
begin
	if __teelib_state.ic <> 1 then
		__teelib_error('Call to getXYM() before init()');
	if __teelib_state.dc <> 0 then
		__teelib_error('Call to getXYM() after done()');
	inc(__teelib_state.gc);

	if (r < 1) or (r > __teelib_state.n) then
		__teelib_error('Invalid call to getXYM()');

	x := __teelib_state.x[r];
	y := __teelib_state.y[r];
	m := __teelib_state.m[r];

	writeln(__teelib_state.fo, 'getXYM(', r, ') -> ', x, ' ', y, ' ', m);
end;

procedure getQ(r : longint; var q : longint);
begin
	if __teelib_state.ic <> 1 then
		__teelib_error('Call to getQ() before init()');
	if __teelib_state.dc <> 0 then
		__teelib_error('Call to getQ() after done()');
	inc(__teelib_state.gc);

	if (r < 1) or (r > __teelib_state.n) then
		__teelib_error('Invalid call to getQ()');
	if __teelib_state.qc[r] >= __teelib_state.m[r] then
		__teelib_error('Too many calls to getQ()');

	inc(__teelib_state.qc[r]);
	q := __teelib_state.q[r, __teelib_state.qc[r]];

	writeln(__teelib_state.fo, 'getQ(', r, ') -> ', q);
end;

procedure put(r : longint);
begin
	if __teelib_state.ic <> 1 then
		__teelib_error('Call to put() before init()');
	if __teelib_state.dc <> 0 then
		__teelib_error('Call to put() after done()');
	if __teelib_state.pc >= __teelib_state.n then
		__teelib_error('Too many calls to put()');

	if (r < 1) or (r > __teelib_state.n) then
		__teelib_error('Invalid call to put()');
	inc(__teelib_state.pc);
	__teelib_state.p[__teelib_state.pc] := r;

	writeln(__teelib_state.fo, 'put(', r, ')');
end;

procedure done();
var i, j, r, q : longint; f : boolean; dx, dy, d :double;
begin
	if __teelib_state.ic <> 1 then
		__teelib_error('Call to done() before init()');
	if __teelib_state.dc <> 0 then
		__teelib_error('Second call to done()');
	inc(__teelib_state.dc);

	r := -1;
	d := 0.0;
	for i := 1 to __teelib_state.pc do begin
		if (i = 1) and (__teelib_state.p[i] <> __teelib_state.s) then
			__teelib_error('Path does not start from S');
		if i > 1 then begin
			f := false;
			for j := 1 to __teelib_state.m[r] do
				if __teelib_state.q[r][j] = __teelib_state.p[i] then
					f := true;
			if not f then
				__teelib_error('Path uses non-existent street');
			q := __teelib_state.p[i];
			dx := __teelib_state.x[r] - __teelib_state.x[q];
			dy := __teelib_state.y[r] - __teelib_state.y[q];
			d := d + sqrt(sqr(dx) + sqr(dy));
		end;
		r := __teelib_state.p[i];
	end;
	if r <> __teelib_state.f then
		__teelib_error('Path does not finish at F');

	writeln(__teelib_state.fo, 'done() -> distance ', d :0:4, ', ', __teelib_state.gc, ' queries');

	close(__teelib_state.fi);
	close(__teelib_state.fo);
end;
